#include "stdafx.h"
#include "OpenGLRendererBase.h"

#pragma comment( lib, "opengl32.lib" )
#pragma comment( lib, "glu32.lib" )

COpenGLRendererBase::COpenGLRendererBase() :
	m_hRC( NULL ),
	m_hDC( NULL ),
	m_cx( 0 ),
	m_cy( 0 )
{
}


COpenGLRendererBase::~COpenGLRendererBase()
{
}

bool COpenGLRendererBase::InitializeOpenGL( HWND hWnd )
{
	// foCXReLXg̍쐬
	m_hDC = GetDC( hWnd );
	if( NULL == m_hDC )
	{
		assert( !"m_pDC is NULL" );
		return false;
	}

	// OpenGL pixel format ̐ݒ
	if( !SetupPixelFormat() )
	{
		assert( !"SetupPixelFormat failed" );
		return false;
	}

	// _OReLXg̍쐬
	m_hRC = wglCreateContext( m_hDC );
	if( 0 == m_hRC )
	{
		assert( !"wglCreateContext failed" );
		return false;
	}

	// _OReLXg̐ݒ
	if( TRUE != wglMakeCurrent( m_hDC, m_hRC ) )
	{
		assert( !"wglMakeCurrent failed" );
		return false;
	}

	// NA
	glClearColor( 0, 0, 0, 0 );	// NAJ[
	glClearDepth( 1.0f );	// NA[x

	// fvX
	glEnable( GL_DEPTH_TEST );	// fvXeXg
	glDepthFunc( GL_LEQUAL );	//fvXt@NiAOɂ̂ŏ`Ăj

	// |S
	glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );	// |SiGL_TRIANGLESj́AʋhԂB
	glDisable( GL_CULL_FACE ); // Ăʂ̃JOiJOȂʂ`j
	glEnable( GL_POLYGON_OFFSET_FILL ); // |SItZbgtBiʂƐdȂꍇ̃JXh~j
	glPolygonOffset( 1.0f, 1.0f ); // |SItZbg

	// CeBO
	glDisable( GL_LIGHTING ); // 
	glEnable( GL_NORMALIZE ); // OpenGLŖ@xNgPʖ@xNg悤ɂ
	glLightModeli( GL_LIGHT_MODEL_TWO_SIDE, 1 );	// ʂ̕\ƗɌ悤ɂi0:\ʂ̂݌B1:ʂɂj
	glDisable( GL_COLOR_MATERIAL ); // J[ݒl}eAƂĎgp

	return true;
}

bool COpenGLRendererBase::SetupPixelFormat()
{
	static PIXELFORMATDESCRIPTOR pfd =
	{
		sizeof( PIXELFORMATDESCRIPTOR ),
		1,
		PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER,
		PFD_TYPE_RGBA,
		24,
		0, 0, 0, 0, 0, 0,
		0,
		0,
		0,
		0, 0, 0, 0,
		32,
		0,
		0,
		PFD_MAIN_PLANE,
		0,
		0, 0, 0
	};

	int iPixelFormat;
	if( 0 == ( iPixelFormat = ChoosePixelFormat( m_hDC, &pfd ) ) )
	{
		assert( !"ChoosePixelFormat is failed" );
		return FALSE;
	}

	if( TRUE != SetPixelFormat( m_hDC, iPixelFormat, &pfd ) )
	{
		assert( !"SetPixelFormat is failed" );
		return FALSE;
	}

	return true;
}

void COpenGLRendererBase::UninitializeOpenGL( HWND hWnd )
{
	wglMakeCurrent( NULL, NULL );
	if( m_hRC )
	{
		wglDeleteContext( m_hRC );
		m_hRC = NULL;
	}
	if( m_hDC )
	{
		ReleaseDC( hWnd, m_hDC );
		m_hDC = NULL;
	}
}

bool COpenGLRendererBase::SetupViewport()
{
	glViewport( 0,
		0,
		m_cx,
		m_cy );
	return true;
}

bool COpenGLRendererBase::SetupProjectionMatrix()
{
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();

	glOrtho( -m_cx * 0.5 / 10.0,
		m_cx * 0.5 / 10.0,
		-m_cy * 0.5 / 10.0,
		m_cy * 0.5 / 10.0,
		0.1,
		1000.0 );

	// eϊsɑ΂鏈͂ŊB
	// ȍ~̕ϊs񏈗́Afr[ϊsɑ΂Ď{悤ɂB
	glMatrixMode( GL_MODELVIEW );
	return true;
}

bool COpenGLRendererBase::SetupModelViewMatrix()
{
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();

	gluLookAt( 0.0, 0.0, 500.0,	// _
		0.0, 0.0, 0.0,	// _
		0.0, 1.0, 0.0 );	// xNg

	return true;
}

void COpenGLRendererBase::ChangeSize( HWND hWnd, int cx, int cy )
{
	if( 0 >= cx || 0 >= cy )
	{
		return;
	}

	if( NULL == m_hRC )
	{
		// OpenGL
		InitializeOpenGL( hWnd );	
	}

	if( NULL != m_hRC )
	{
		// ʑ傫̐ݒ
		m_cx = cx;
		m_cy = cy;

		// r[|[gݒ菈
		SetupViewport();

		// eϊs̐ݒ
		SetupProjectionMatrix();

		// fOr[COϊs̐ݒ
		SetupModelViewMatrix();
	}
}

void COpenGLRendererBase::Render()
{
	if( !m_hDC )
	{
		return;
	}

	// _OReLXg̐ݒ
	// MDI`ɂĕK{BSDIɂĂ͏ȗ(InitializeOpenGL֐ŌĂяoς݂Ȃ̂ŁjB
	// ȍ~OpenGL֐ɂ`́Aw肵_OReLXɕ`悳B
	wglMakeCurrent( m_hDC, m_hRC );

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	PreRenderScene();

	glPushMatrix();
	RenderStockScene();
	glPopMatrix();

	glPushMatrix();
	RenderScene();
	glPopMatrix();

	PostRenderScene();

	glFinish();

	SwapBuffers( m_hDC );
}

void COpenGLRendererBase::RenderStockScene()
{
	RenderAxis();
}

void COpenGLRendererBase::RenderAxis()
{
	float	points[4][3];
	points[0][0] = 0.0;
	points[0][1] = 0.0;
	points[0][2] = 0.0;
	points[1][0] = 10;
	points[1][1] = 0.0;
	points[1][2] = 0.0;
	points[2][0] = 0.0;
	points[2][1] = 10;
	points[2][2] = 0.0;
	points[3][0] = 0.0;
	points[3][1] = 0.0;
	points[3][2] = 10;

	glPushAttrib( GL_CURRENT_BIT | GL_LINE_BIT );	// F | 

	glLineWidth( 2.0 );
	glBegin( GL_LINES );
	glColor3ub( 255, 0, 0 );
	glVertex3fv( points[0] );
	glVertex3fv( points[1] );
	glColor3ub( 0, 255, 0 );
	glVertex3fv( points[0] );
	glVertex3fv( points[2] );
	glColor3ub( 0, 0, 255 );
	glVertex3fv( points[0] );
	glVertex3fv( points[3] );
	glEnd();

	glPopAttrib();
}
